@@ -5,9 +5,11 @@ class PluginAlreadyRegistered(Exception):
 class PluginNotRegistered(Exception):
     pass
 
-class AppAllreadyRegistered(Exception):
+class AppAlreadyRegistered(Exception):
     pass
 
+AppAllreadyRegistered = AppAlreadyRegistered # backwards compatibility, will be dropped in 2.3
+
 class NotImplemented(Exception):
     pass
 
@@ -29,6 +31,8 @@ class NoPermissionsException(PermissionsException):
     """Can be fired when some violate action is performed on permission system. 
     """
     
+class Deprecated(Exception): pass
+    
 class DuplicatePlaceholderWarning(Warning): pass
 
 class DontUsePageAttributeWarning(Warning): pass
\ No newline at end of file
