@@ -1,25 +1,35 @@
 # -*- coding: utf-8 -*-
 from __future__ import with_statement
 from cms.admin.forms import PageForm
-from cms.api import create_page
+from cms.api import create_page, add_plugin
 from cms.models import Page, Title
 from cms.models.placeholdermodel import Placeholder
 from cms.models.pluginmodel import CMSPlugin
+from cms.plugins.link.cms_plugins import LinkPlugin
+from cms.plugins.text.cms_plugins import TextPlugin
 from cms.plugins.text.models import Text
 from cms.sitemaps import CMSSitemap
-from cms.test_utils.testcases import (CMSTestCase, URL_CMS_PAGE, 
+from cms.templatetags.cms_tags import get_placeholder_content
+from cms.test_utils.testcases import (CMSTestCase, URL_CMS_PAGE,
+                                      URL_CMS_PAGE_ADD)
+from cms.test_utils.util.context_managers import (LanguageOverride,
+                                                  SettingsOverride)
+from cms.utils.page_resolver import get_page_from_request
+from cms.test_utils.testcases import (CMSTestCase, URL_CMS_PAGE,
     URL_CMS_PAGE_ADD)
 from cms.test_utils.util.context_managers import (LanguageOverride, 
     SettingsOverride)
-from cms.utils.page_resolver import get_page_from_request
+from cms.utils.page_resolver import get_page_from_request, is_valid_url
 from django.conf import settings
 from django.contrib.sites.models import Site
+from django.core.exceptions import ValidationError
 from django.core.urlresolvers import reverse
 from django.http import HttpRequest, HttpResponse, HttpResponseNotFound
 import datetime
 import os.path
 from cms.utils.page import is_valid_page_slug
 
+
 class PagesTestCase(CMSTestCase):
     
     def test_add_page(self):
@@ -163,7 +173,34 @@ class PagesTestCase(CMSTestCase):
             response = self.client.post('/admin/cms/page/%s/' %page.id, page_data)
             self.assertRedirects(response, URL_CMS_PAGE)
             self.assertEqual(page.get_title(), 'changed title')
-    
+
+    def test_moderator_edit_page_redirect(self):
+        """
+        Test that a page can be edited multiple times with moderator
+        """
+        superuser = self.get_superuser()
+        with self.login_user_context(superuser):
+            with SettingsOverride(CMS_MODERATOR=True):
+                page_data = self.get_new_page_data()
+                response = self.client.post(URL_CMS_PAGE_ADD, page_data)
+                self.assertEquals(response.status_code, 302)
+                page =  Page.objects.get(title_set__slug=page_data['slug'])
+                response = self.client.get('/en/admin/cms/page/%s/' %page.id)
+                self.assertEqual(response.status_code, 200)
+                page_data['overwrite_url'] = '/hello/'
+                page_data['has_url_overwrite'] = True
+                response = self.client.post('/en/admin/cms/page/%s/' %page.id, page_data)
+                self.assertRedirects(response, URL_CMS_PAGE)
+                self.assertEqual(page.get_absolute_url(), '/hello/')
+                title = Title.objects.all()[0]
+                page.publish()
+                page_data['title'] = 'new title'
+                response = self.client.post('/en/admin/cms/page/%s/' %page.id, page_data)
+                page =  Page.objects.get(title_set__slug=page_data['slug'], publisher_is_draft=True)
+                self.assertRedirects(response, URL_CMS_PAGE)
+                self.assertEqual(page.get_title(), 'new title')
+
+
     def test_meta_description_and_keywords_fields_from_admin(self):
         """
         Test that description and keywords tags can be set via the admin
@@ -329,6 +366,26 @@ class PagesTestCase(CMSTestCase):
                     published=True, in_navigation=True)
         self.assertEqual(CMSSitemap().items().count(),0)
 
+    def test_sitemap_includes_last_modification_date(self):
+        one_day_ago = datetime.datetime.now() - datetime.timedelta(days=1)
+        page = create_page("page", "nav_playground.html", "en", published=True, publication_date=one_day_ago)
+        page.creation_date = one_day_ago
+        page.save()
+        sitemap = CMSSitemap()
+        self.assertEqual(sitemap.items().count(), 1)
+        actual_last_modification_time = sitemap.lastmod(sitemap.items()[0])
+        self.assertTrue(actual_last_modification_time > one_day_ago)
+
+    def test_sitemap_uses_publication_date_when_later_than_modification(self):
+        now = datetime.datetime.now()
+        one_day_ago = now - datetime.timedelta(days=1)
+        page = create_page("page", "nav_playground.html", "en", published=True, publication_date=now)
+        page.creation_date = one_day_ago
+        page.changed_date = one_day_ago
+        sitemap = CMSSitemap()
+        actual_last_modification_time = sitemap.lastmod(page)
+        self.assertEqual(actual_last_modification_time, now)
+
     def test_edit_page_other_site_and_language(self):
         """
         Test that a page can edited via the admin when your current site is
@@ -378,10 +435,22 @@ class PagesTestCase(CMSTestCase):
         """
         parent = create_page("parent", "nav_playground.html", "en")
         child = create_page("child", "nav_playground.html", "en", parent=parent)
+        grand_child = create_page("child", "nav_playground.html", "en", parent=child)
         child.template = settings.CMS_TEMPLATE_INHERITANCE_MAGIC
+        grand_child.template = settings.CMS_TEMPLATE_INHERITANCE_MAGIC
         child.save()
-        self.assertEqual(child.template, settings.CMS_TEMPLATE_INHERITANCE_MAGIC)
-        self.assertEqual(parent.get_template_name(), child.get_template_name())
+        grand_child.save()
+
+        # kill template cache
+        delattr(grand_child, '_template_cache')
+        with self.assertNumQueries(1):
+            self.assertEqual(child.template, settings.CMS_TEMPLATE_INHERITANCE_MAGIC)
+            self.assertEqual(parent.get_template_name(), grand_child.get_template_name())
+
+        # test template cache
+        with self.assertNumQueries(0):
+            grand_child.get_template()
+
         parent.template = settings.CMS_TEMPLATE_INHERITANCE_MAGIC
         parent.save()
         self.assertEqual(parent.template, settings.CMS_TEMPLATE_INHERITANCE_MAGIC)
@@ -599,6 +668,43 @@ class PagesTestCase(CMSTestCase):
         self.assertEqual(page3.get_absolute_url(),
             self.get_pages_root()+'i-want-another-url/')
 
+    def test_slug_url_overwrite_clash(self):
+        """ Tests if a URL-Override clashes with a normal page url
+        """
+        with SettingsOverride(CMS_MODERATOR=False, CMS_PERMISSION=False):
+            home = create_page('home', 'nav_playground.html', 'en', published=True)
+            bar = create_page('bar', 'nav_playground.html', 'en', published=False)
+            foo = create_page('foo', 'nav_playground.html', 'en', published=True)
+            # Tests to assure is_valid_url is ok on plain pages
+            self.assertTrue(is_valid_url(bar.get_absolute_url('en'),bar))
+            self.assertTrue(is_valid_url(foo.get_absolute_url('en'),foo))
+
+            # Set url_overwrite for page foo
+            title = foo.get_title_obj(language='en')
+            title.has_url_overwrite = True
+            title.path = '/bar/'
+            title.save()
+            try:
+                url = is_valid_url(bar.get_absolute_url('en'),bar)
+            except ValidationError:
+                url = False
+            if url:
+                bar.published = True
+                bar.save()
+            self.assertFalse(bar.published)
+
+    def test_valid_url_multisite(self):
+        with SettingsOverride(CMS_MODERATOR=False):
+            site1 = Site.objects.get_current()
+            site3 = Site.objects.create(domain="sample3.com", name="sample3.com")
+            home = create_page('home', 'nav_playground.html', 'en', published=True, site=site1)
+            bar = create_page('bar', 'nav_playground.html', 'en', slug="bar", published=True, parent=home, site=site1)
+            home_s3= create_page('home', 'nav_playground.html', 'en', published=True, site=site3)
+            bar_s3 = create_page('bar', 'nav_playground.html', 'en', slug="bar", published=True, parent=home_s3, site=site3)
+
+            self.assertTrue(is_valid_url(bar.get_absolute_url('en'), bar))
+            self.assertTrue(is_valid_url(bar_s3.get_absolute_url('en'), bar_s3))
+
     def test_home_slug_not_accessible(self):
         with SettingsOverride(CMS_MODERATOR=False, CMS_PERMISSION=False):
             page = create_page('page', 'nav_playground.html', 'en', published=True)
@@ -625,6 +731,28 @@ class PagesTestCase(CMSTestCase):
         self.assertEqual(Page.objects.drafts().get_home().get_slug(), 'home')
         self.assertEqual(Page.objects.public().get_home().get_slug(), 'home')
 
+    def test_plugin_loading_queries(self):
+        with SettingsOverride(CMS_TEMPLATES = (('placeholder_tests/base.html', 'tpl'),)):
+            page = create_page('home', 'placeholder_tests/base.html', 'en', published=True, slug='home')
+            placeholders = list(page.placeholders.all())
+            for i, placeholder in enumerate(placeholders):
+                for j in range(5):
+                    add_plugin(placeholder, TextPlugin, 'en', body='text-%d-%d' % (i, j))
+                    add_plugin(placeholder, LinkPlugin, 'en', name='link-%d-%d' % (i, j))
+            from django.db import connection
+            connection.queries = []
+
+            # trigger the apphook query so that it doesn't get in our way
+            reverse('pages-root')
+            with self.assertNumQueries(4):
+                context = self.get_context()
+                for i, placeholder in enumerate(placeholders):
+                    content = get_placeholder_content(context, context['request'], page, placeholder.slot, False)
+                    for j in range(5):
+                        self.assertIn('text-%d-%d' % (i, j), content)
+                        self.assertIn('link-%d-%d' % (i, j), content)
+
+
 class NoAdminPageTests(CMSTestCase):
     urls = 'cms.test_utils.project.noadmin_urls'
     
